# -*- coding: utf-8 -*-

import logging
from django.contrib import admin, messages
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from jmb.core.utils import models_utils


logger = logging.getLogger('celery.task')

def clone_car_items(modeladmin, request, queryset):
    done_count = 0
    kw = {
        'kwargs_excludes': [
            'date_create', 'date_last_modify', 'ordering',
            'creator', 'last_modifier',
        ]
    }
    for car in queryset.all():
        done, msg, new_car_item = models_utils.ModelObjectCopier(car, related_parent_name=None, **kw).bulk_copy()
        if done:
            done_count += 1
    messages.info(request, _("Created {} new cars".format(done_count)))
    return redirect(request.META.get('HTTP_REFERER'))
clone_car_items.short_description = _("Clone car items")


def refresh_car_items_slug(modeladmin, request, queryset):
    from publieditor.content.models import Car
    done_count = 0
    for car_item in queryset:
        slug = slugify(car_item.version)
        new_slug = slug
        cont = 0
        while True:
            if Car.objects.filter(slug=new_slug).exclude(id=car_item.id).count():
                cont += 1
                new_slug = "%s-%s" % (slug, cont)
            else:
                break
        car_item.slug = new_slug
        car_item.save()
    messages.info(request, _("Items slug refreshed"))
    return redirect(request.META.get('HTTP_REFERER'))
refresh_car_items_slug.short_description = _("Refresh items slug")


def car_async_csv_export(modeladmin, request, queryset):
    from publieditor.content.tasks import CarAsyncCsvExportTask
    request_get = request.GET.copy()
    pks = request.POST.getlist('_selected_action')
    select_across = int(request.POST.get('select_across'))
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    CarAsyncCsvExportTask.delay_or_fail(
        app_label='content', model=modeladmin.model,
        pks=pks, select_across=select_across, get=request_get
    )
    return redirect(reverse("admin:content_car_changelist")+"?{}".format(request.GET.urlencode()))
car_async_csv_export.short_description = _("CSV async export")


def car_async_xls_export(modeladmin, request, queryset):
    from publieditor.content.tasks import CarAsyncXlsExportTask
    request_get = request.GET.copy()
    pks = request.POST.getlist('_selected_action')
    select_across = int(request.POST.get('select_across'))
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    CarAsyncXlsExportTask.delay_or_fail(
        app_label='content', model=modeladmin.model,
        pks=pks, select_across=select_across, get=request_get
    )
    return redirect(reverse("admin:content_car_changelist")+"?{}".format(request.GET.urlencode()))
car_async_xls_export.short_description = _("XLS async export")


def send_issue_emails(modeladmin, request, queryset):
    logger.info("send_issue_emails t1")
    from publieditor.content.tasks import SendContractEmailsTask
    if queryset.count() > 1:
        modeladmin.message_user(request, _("You must select just one issue!"))
        return redirect(reverse("admin:content_issue_changelist"))
    issue_email_template_id = int(request.POST.get("issue_email_template"))
    organization_groups_id = [int(id) for id in request.POST.getlist("organization_group")]
    owner = request.POST.get("owner")
    request_get = request.GET.copy()
    request_request = request.REQUEST.copy()
    select_across = int(request.POST.get("select_across"))
    pks = request.POST.getlist('_selected_action')
    send_to_all = request.POST.get("send_to_all")
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    logger.info("send_issue_emails t2")
    logger.info("issue_email_template_id {}".format(issue_email_template_id))
    logger.info("organization_groups_id {}".format(organization_groups_id))
    logger.info("owner {}".format(owner))
    logger.info("pks {}".format(pks))
    SendContractEmailsTask.delay_or_fail(
        app_label='content',
        model=modeladmin.model,
        request_get=request_get,
        request_request=request_request,
        issue_email_template_id=issue_email_template_id,
        organization_groups_id=organization_groups_id,
        owner=owner,
        pks=pks,
        select_across=select_across,
        send_to_all=send_to_all
    )
    logger.info("send_issue_emails t3")
    return redirect(reverse("admin:content_issue_changelist"))
send_issue_emails.short_description = _('Send issue emails')


def content_async_csv_export(modeladmin, request, queryset):
    from publieditor.content.tasks import ContentAsyncCsvExportTask
    request_get = request.GET.copy()
    pks = request.POST.getlist('_selected_action')
    select_across = int(request.POST.get('select_across'))
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    ContentAsyncCsvExportTask.delay_or_fail(
        app_label='content', model=modeladmin.model,
        pks=pks, select_across=select_across, get=request_get
    )
    return redirect(reverse("admin:content_content_changelist")+"?{}".format(request.GET.urlencode()))
content_async_csv_export.short_description = _("CSV async export")


def content_async_xls_export(modeladmin, request, queryset):
    from publieditor.content.tasks import ContentAsyncXlsExportTask
    request_get = request.GET.copy()
    pks = request.POST.getlist('_selected_action')
    select_across = int(request.POST.get('select_across'))
    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
    ContentAsyncXlsExportTask.delay_or_fail(
        app_label='content', model=modeladmin.model,
        pks=pks, select_across=select_across, get=request_get
    )
    # cacet = ContentAsyncXlsExportTask()
    # cacet.run(model=modeladmin.model,
    #      pks=pks, select_across=select_across, get=request_get)
    return redirect(reverse("admin:content_content_changelist")+"?{}".format(request.GET.urlencode()))
content_async_xls_export.short_description = _("XLS async export")