# -*- coding: utf-8 -*-

import base64
import json
from dajax.core import Dajax
from dajaxice.decorators import dajaxice_register
from django.template.defaultfilters import slugify

from publieditor.content.models import Brand, CarModel, Car, TranslationContent, CarEquipment


@dajaxice_register
def on_select_corporate(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_brand', 'innerHTML', options)
    dajax.assign('#id_model', 'innerHTML', options)
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    if not id_corporate:
        return dajax.json()

    brands = Brand.objects.filter(corporate_id=id_corporate).distinct()
    for brand in brands:
        options += "<option value='%s'>%s</option>" % (
            brand.id, brand.name
        )
    dajax.assign('#id_brand', 'innerHTML', options)
    return dajax.json()

@dajaxice_register
def on_select_corporate_news(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_brand1', 'innerHTML', options)
    dajax.assign('#id_brand2', 'innerHTML', options)
    dajax.assign('#id_model', 'innerHTML', options)
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    if not id_corporate:
        return dajax.json()

    brands = Brand.objects.filter(corporate_id=id_corporate).distinct()
    for brand in brands:
        options += "<option value='%s'>%s</option>" % (
            brand.id, brand.name
        )
    dajax.assign('#id_brand1', 'innerHTML', options)
    dajax.assign('#id_brand2', 'innerHTML', options)
    return dajax.json()

@dajaxice_register
def on_select_brand(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_model', 'innerHTML', options)
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    id_brand = kwargs.get('id_brand', None)
    if not id_brand:
        return dajax.json()

    models = CarModel.objects.filter(brand_id=id_brand)
    if id_corporate:
        models = models.filter(brand__corporate_id=id_corporate)
    for model in models:
        options += "<option value='%s'>%s</option>" % (
            model.id, model.name
        )
    dajax.assign('#id_model', 'innerHTML', options)
    return dajax.json()


@dajaxice_register
def on_select_model(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    id_brand = kwargs.get('id_brand', None)
    id_model = kwargs.get('id_model', None)
    if not id_model:
        return dajax.json()

    cars = Car.objects.filter(model_id=id_model)
    if id_brand:
        cars = cars.filter(model__brand_id=id_brand)
    if id_corporate:
        cars = cars.filter(model__brand__corporate=id_corporate)
    for car in cars:
        options += "<option value='%s'>%s</option>" % (
            car.id, car.version
        )
    dajax.assign('#id_car', 'innerHTML', options)
    return dajax.json()


@dajaxice_register
def on_click_btn_make_slug(request, *args, **kwargs):
    dajax = Dajax()
    html_id = kwargs.get('html_id', None)
    content_translation_id = kwargs.get('content_translation_id', None)
    title = kwargs.get('title', None)
    if content_translation_id and title:
        translation_slug = slugify(title)
        new_slug = translation_slug
        cont = 0
        while True:
            if TranslationContent.objects.filter(slug=new_slug).exclude(id=content_translation_id).count():
                cont += 1
                new_slug = "%s-%s" % (translation_slug, cont)
            else:
                break
        dajax.assign('#id_translations-%s-slug' % html_id, 'value', new_slug)
    return dajax.json()


@dajaxice_register
def on_click_car_make_slug(request, *args, **kwargs):
    dajax = Dajax()
    car_id = kwargs.get('car_id', None)
    car_id = car_id if car_id.isdigit() else None
    car_version = kwargs.get('car_version', None)
    if car_version:
        slug = slugify(car_version)
        new_slug = slug
        cont = 0
        while True:
            query = Car.objects.filter(slug=new_slug)
            if car_id:
                query = query.exclude(id=car_id)
            if query.count():
                cont += 1
                new_slug = "%s-%s" % (slug, cont)
            else:
                break
        dajax.assign('#id_slug', 'value', new_slug)
    return dajax.json()


@dajaxice_register
def update_equiment_matrix_cell(request, *args, **kwargs):
    dajax = Dajax()
    value = kwargs.get('value')
    equipment_id = kwargs.get('equipment')
    car_id = kwargs.get('car')
    query_filter = {'car_id': car_id, 'carmodel_equipment_id': equipment_id}
    record_exists = CarEquipment.objects.filter(**query_filter).exists()
    if not value:
        CarEquipment.objects.get(**query_filter).delete()
    elif value and not record_exists:
        car_equipment = CarEquipment(**query_filter)
        car_equipment.save()
    dajax.add_css_class('#loader', 'invisible')
    return dajax.json()


@dajaxice_register
def update_equiment_matrix_row(request, *args, **kwargs):
    dajax = Dajax()
    value = kwargs.get('value')
    equipment_id = kwargs.get('equipment')
    cars = json.loads(kwargs.get('cars'))
    carmodel_id = kwargs.get('carmodel')
    carmodel = CarModel.objects.get(id=int(carmodel_id))
    if not value:
        qs = CarEquipment.objects.filter(car__model=carmodel, carmodel_equipment_id=equipment_id)
        if len(cars):
            qs = qs.filter(car_id__in=cars)
        qs.delete()
    else:
        create_objects = []
        cars_qs = carmodel.cars.filter(status__in=('ok', 'working'))
        if len(cars):
            cars_qs = cars_qs.filter(id__in=cars)
        for car in cars_qs:
            query_filter = {'car_id': car.id, 'carmodel_equipment_id': equipment_id}
            if not CarEquipment.objects.filter(**query_filter).exists():
                create_objects.append(CarEquipment(**query_filter))
        if create_objects:
            CarEquipment.objects.bulk_create(create_objects)
    dajax.add_css_class('#loader', 'invisible')
    return dajax.json()


@dajaxice_register
def update_equiment_matrix_column(request, *args, **kwargs):
    dajax = Dajax()
    value = kwargs.get('value')
    car_id = kwargs.get('car')
    car = Car.objects.get(id=car_id)
    if not value:
        CarEquipment.objects.filter(car_id=car_id).delete()
    else:
        create_objects = []
        for carmodel_equipment in car.model.equipments.all():
            query_filter = {'car_id': car_id, 'carmodel_equipment_id': carmodel_equipment.id}
            if not CarEquipment.objects.filter(**query_filter).exists():
                create_objects.append(CarEquipment(**query_filter))
        if create_objects:
            CarEquipment.objects.bulk_create(create_objects)
    dajax.add_css_class('#loader', 'invisible')
    return dajax.json()


@dajaxice_register
def update_equiment_matrix_group(request, *args, **kwargs):
    dajax = Dajax()
    value = kwargs.get('value')
    motor = base64.b64decode(kwargs.get('motor'))
    carmodel_id = kwargs.get('carmodel')
    carmodel = CarModel.objects.get(id=int(carmodel_id))
    if not value:
        CarEquipment.objects.filter(car__model=carmodel, car__motor=motor).delete()
    else:
        create_objects = []
        for carmodel_equipment in carmodel.equipments.all():
            for model_car in carmodel.cars.filter(status__in=('ok', 'working'), motor=motor).filter():
                query_filter = {'car_id': model_car.id, 'carmodel_equipment_id': carmodel_equipment.id}
                if not CarEquipment.objects.filter(**query_filter).exists():
                    create_objects.append(CarEquipment(**query_filter))
        if create_objects:
            CarEquipment.objects.bulk_create(create_objects)
    dajax.add_css_class('#loader', 'invisible')
    return dajax.json()

