# -*- coding: utf-8 -*-

import autocomplete_light
import os
import csv
import xlwt
import logging
import datetime
from django.contrib import admin
from django.conf import settings
from django.http import HttpRequest
from django.utils.translation import ugettext as _

from jmb.async.models import Task
from jmb.async.tasks import GenericJmbJobtasticTask
from jmb.async.tasks import JmbJobtasticTask
from publieditor.content.utils import _export_data
autocomplete_light.autodiscover()

logger = logging.getLogger('celery.task')

CAR_LIST_DISPLAY_CSV = (
        'id', 'status', 'infocar', 'get_corporate_id', 'get_corporate_name', 'get_brand_id', 'get_brand_name',
        'get_model_id', 'get_model_name', 'get_model_status', 'motor', 'version', 'price', 'shape',
        'target', 'seat', 'engine', 'layout', 'turbo', 'fuel', 'traction', 'gear', 'agear', 'pollution',
        'engine_loc', 'doors', 'year_first', 'year', 'month', 'luggage', 'luggage_max',
        'tank', 'weight', 'length', 'width', 'height', 'displ', 'power_cv', 'power_kw', 'power_rpm',
        'torque_max', 'torque_rpm', 'speed', 'acc', 'co2', 'tow', 'cons_urban', 'cons_mix', 'cons_extra',
        'turning_circle', 'warranty', 'wheelbase', 'kml_mix', 'get_neopatentati', 'get_low_emission',
        'get_area_c', 'power_spec', 'power_weight', 'proxymity', 'get_price_update', 'field_src',
        'get_new_date', 'get_mod_date', 'standard_num', 'off_year', 'off_month', 'slug',
        'get_autocomposed_text'
    )


class CarAsyncCsvExportTask(JmbJobtasticTask):
    task_name = _("Content: Car csv export")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        if select_across:
            request = HttpRequest()
            request.GET = get
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        rows = []
        for count, row in enumerate(_export_data(modeladmin, CAR_LIST_DISPLAY_CSV, queryset=qs)):
            rows += [row]
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
        task_id = self.request.id
        output_filename = self.create_file(
            rows, output_filename=os.path.join(settings.MEDIA_ROOT, 'tasks', "%s.csv" % task_id)
        )
        task = Task.objects.get(task_id=task_id)
        msg = 'exported {} records'.format(count)
        task.message = msg
        task.url = "tasks/{}.xls".format(task_id)
        task.save()
        return {'message': msg, 'url': output_filename}

    @staticmethod
    def create_file(rows, output_filename=None):
        encoded_rows = []
        for row in rows:
            encoded_rows.append([value.encode("utf-8") for value in row])
        with open(output_filename, 'wb') as writer:
            out = csv.writer(writer, delimiter=';', quotechar='"', quoting=csv.QUOTE_ALL)
            out.writerows(encoded_rows)
        return output_filename


class CarAsyncXlsExportTask(JmbJobtasticTask):
    task_name = _("Content: Car xls export")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        if select_across:
            request = HttpRequest()
            request.GET = get
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        rows = []
        for count, row in enumerate(_export_data(modeladmin, CAR_LIST_DISPLAY_CSV, queryset=qs)):
            rows += [row]
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
        task_id = self.request.id
        output_filename = self.create_file(
            rows, output_filename=os.path.join(settings.MEDIA_ROOT, 'tasks', "%s.xls" % task_id)
        )
        task = Task.objects.get(task_id=task_id)
        msg = 'exported {} records'.format(count)
        task.message = msg
        task.url = "tasks/{}.xls".format(task_id)
        task.save()
        return {'message': msg, 'url': output_filename}

    @staticmethod
    def create_file(rows, output_filename):
        wb = xlwt.Workbook()
        ws = wb.add_sheet("Cars export")
        for row_num, row in enumerate(rows):
            for col_num, value in enumerate(row):
                ws.write(row_num, col_num, value)
        wb.save(output_filename)
        return output_filename


class SendContractEmailsTask(GenericJmbJobtasticTask):
    task_name = _("Content: Send contracts emails")

    def calculate_result(
        self, app_label=None, model=None, request_get=None,
        request_request=None, select_across=None, pks=None,
        **kwargs
        ):
        logger.info('SendContractEmailsTask.calculate_result t1')
        self.set_init_calculate_result(model, request_get, request_request, select_across, pks, **kwargs)
        for count, qs_obj in enumerate(self.queryset):
            logger.info('SendContractEmailsTask.calculate_result for')
            self.do_action(qs_obj)
            self.update_progress(
                count,
                self.queryset.count(),
                update_frequency=100,
                )
        logger.info('SendContractEmailsTask.calculate_result post-for')

        self.set_messages()
        self.set_task()
        logger.info('SendContractEmailsTask.calculate_result pre-return')

        return self.task_name.capitalize()
                                                            
    def set_report_msg_dict(self):
        self.report_msg_dict = dict(
            success_msg_0=_("zero emails sent"),
            success_msg_1=_("1 email sent"),
            success_msg_n="{} emails sent for issue n. {} ({})".format(
                self.success_count,
                self.issue.number,
                self.issue.pub_date
            ),
            error_msg_1=_("1 error found"),
            error_msg_n=_("{} errors found".format(self.error_count))
        )

    def do_action(self, issue):
        from jmb.organization.models import Contact
        from publieditor.contract.models import Contract
        from publieditor.contract.models import IssueEmailTemplate
        self.issue = issue
        logger.info("SendContractEmailsTask.do_action start")
        issue_email_template_id = self.kwargs.get("issue_email_template_id")
        organization_groups_id = self.kwargs.get("organization_groups_id")
        owner = self.kwargs.get("owner")
        template = IssueEmailTemplate.objects.get(id=issue_email_template_id)
        send_to_all = self.kwargs.get("send_to_all")
        logger.info(
            "SendContractEmailsTask.do_action "
            "template_id:{}, "
            "organization_groups_id:{}, "
            "owner:{}, send_to_all:{}"
            "".format(issue_email_template_id, organization_groups_id, owner, send_to_all))
        if send_to_all:
            logger.info("SendContractEmailsTask.do_action - send to all")
            contacts = Contact.objects.filter(status=1, email__isnull=False, organization__owner=owner).exclude(email='')
            if organization_groups_id:
                contacts = contacts.filter(contact_groups__id__in=organization_groups_id)
            result, msg = issue.send_contract_email(template, contacts, is_test=False)
            if result:
                self.success_count += contacts.count()
            else:
                self.error_msg.append(msg)
                self.error_count += 1
        else:
            today = datetime.date.today()
            active_contracts = Contract.objects.filter(date_begin__lte=today, date_end__gte=today)
            logger.info("SendContractEmailsTask.do_action - send enabled contacts")
            for contract in active_contracts:
                logger.info("SendContractEmailsTask.do_action - send enabled contacts.for", contract.organization)
                contacts = contract.organization.contacts.filter(status=1, email__isnull=False, organization__owner=owner).exclude(email='')
                if organization_groups_id:
                    contacts = contacts.filter(contact_groups__id__in=organization_groups_id)
                result, msg = issue.send_contract_email(template, contacts, is_test=False)
                if result:
                    self.success_count += contacts.count()
                else:
                    self.error_msg.append(msg)
                    self.error_count += 1
        logger.info("SendContractEmailsTask.do_action end")
        logger.info("success_count: {}, error_count: {}".format(self.success_count, self.error_count))


CONTENT_LIST_DISPLAY_CSV = (
    'id', 'parent', 'get_date_content', 'status', 'layout', 'stream',
    'issue', 'code', 'page', 'note', 'sector', 'kind',
    'get_current_language', 'get_title_by_language',
    'get_description_by_language', 'get_abstract_by_language',
    'get_body_by_language', 'get_has_photo', 'get_has_table', 'get_has_video',
    'get_first_photo_url', 'get_first_table_url',
)


class ContentAsyncCsvExportTask(JmbJobtasticTask):
    task_name = _("Content: Content csv export")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1
    _last_update_count = 0

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        if select_across:
            request = HttpRequest()
            request.GET = get
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        rows = []
        for count, row in enumerate(_export_data(modeladmin, CONTENT_LIST_DISPLAY_CSV, queryset=qs)):
            rows += [row]
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
        task_id = self.request.id
        output_filename = self.create_file(
            rows, output_filename=os.path.join(settings.MEDIA_ROOT, 'tasks', "%s.csv" % task_id)
        )
        task = Task.objects.get(task_id=task_id)
        msg = 'exported {} records'.format(count)
        task.message = msg
        task.url = "tasks/{}.csv".format(task_id)
        task.save()
        return {'message': msg, 'url': output_filename}

    @staticmethod
    def create_file(rows, output_filename=None):
        encoded_rows = []
        for row in rows:
            encoded_rows.append([value.encode("utf-8") for value in row])
        with open(output_filename, 'wb') as writer:
            out = csv.writer(writer, delimiter=';', quotechar='"', quoting=csv.QUOTE_ALL)
            out.writerows(encoded_rows)
        return output_filename


class ContentAsyncXlsExportTask(JmbJobtasticTask):
    task_name = _("Content: Content xls export")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def calculate_result(self, model=None, pks=None, select_across=None, get=None):
        admin.autodiscover()
        modeladmin = admin.site._registry[model]
        if select_across:
            request = HttpRequest()
            request.GET = get
            qs = modeladmin.queryset(request)
            n_of_steps = qs.count()
        else:
            qs = modeladmin.model.objects.filter(id__in=pks)
            n_of_steps = len(pks)
        rows = []
        for count, row in enumerate(_export_data(modeladmin, CONTENT_LIST_DISPLAY_CSV, queryset=qs)):
            rows += [row]
            self.update_progress(
                count,
                n_of_steps,
                update_frequency=100,
            )
        task_id = self.request.id
        output_filename = self.create_file(
            rows, output_filename=os.path.join(settings.MEDIA_ROOT, 'tasks', "%s.xls" % task_id)
        )
        task = Task.objects.get(task_id=task_id)
        msg = 'exported {} records'.format(count)
        task.message = msg
        task.url = "tasks/{}.xls".format(task_id)
        task.save()
        return {'message': msg, 'url': output_filename}

    @staticmethod
    def create_file(rows, output_filename):
        wb = xlwt.Workbook()
        ws = wb.add_sheet("Contents export")
        for row_num, row in enumerate(rows):
            for col_num, value in enumerate(row):
                ws.write(row_num, col_num, value)
        wb.save(output_filename)
        return output_filename
